---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../../src
---

# El mundo de las cintas

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Considérese el lenguaje $L$ que posee tres predicados de aridad 1 $\{C, T, E\}$, dos símbolos de predicado de aridad 2, $I$ y $D$, dos símbolos de constante $\{a, b\}$ y dos símbolos de función de aridad 1.

En este ejercicio, una cinta es una $L$-estructura para el lenguaje L cuyo universo puede ser descrito por una lista (posiblemente infinita) de figuras (cuadrados, triángulos y estrellas) y la interpretación de los símbolos de predicado es la natural si suponemos que $C(x)$ expresa “_x es un cuadrado_”, $T(x)$ expresa “_x es un triángulo_”, $E(x)$ expresa “_x es una estrella_”, $I(x, y)$ expresa “_x está a la izquierda de y_” y $D(x, y)$ expresa “_x está a la derecha de y_”.

$\boxed{\textit{Apartado 1}}$. Formalizar los siguientes enunciados:

- $\psi_1$: Toda estrella tiene algún cuadrado a la derecha.

- $\psi_2$: Si a y b son dos posiciones de la cinta (no necesariamente distintas) entonces hay algún elemento que está a la izquierda de ambos y que no es un triángulo. 

- $\psi_3$: En la cinta aparece la secuencia 	■ ★ ▲

$\boxed{\textit{Apartado 2}}$. Estudia las instancias libres, ligadas y el árbol de formación de la fórmula:

$$ \forall x \,\exists y \,\left( I(x,y) \rightarrow \left( D(z,x) \vee \exists z \,\left( D(z,y) \wedge I(x,y) \right) \right) \right) $$

- ¿Es una fórmula bien formada?¿Es posible realizar un renombramiento de las variables tal que no existan variables libres y ligadas al mismo tiempo?

- A qué formula correspondería la clausura (universal) de la fórmula.


$\boxed{\textit{Apartado 3}}$. Dadas las siguientes tres cintas:

$$ \begin{array}{c c c c c c} \hline M1 \hspace{0.5cm} & \Box & \Box &  \star  & \triangle & \Box \\ \hline & a & & & b & \end{array}$$

$$ \begin{array}{c c c c c c c} \hline M2 \hspace{0.5cm} & \triangle & \star &  \triangle  & \Box & \triangle & \triangle \\ \hline & & & & b & a &  \end{array}$$

$$ \begin{array}{c c c c c} \hline M3 \hspace{0.5cm} & \Box & \Box &  \Box  &  \Box \\ \hline & & \begin{matrix} a \\ b \end{matrix} \end{array}$$

- Define las L-estructuras asociadas a las mismas

- Para cada una de las siguientes fórmulas establece si alguna de las 3 L-estructuras es modelo de la fórmula: $\varphi_1 \equiv C(a)  \vee (\neg E(b) \wedge (T(b) \rightarrow \exists x C(x)))$; $\varphi_2 \equiv \forall x \left( C(x) \rightarrow \exists y (T(y) \wedge I(y,x)) \right)$


## Solución

Antes de nada, vamos a importar el módulo de Sintaxis y Semántica en Primer Orden y algunos otros paquetes que nos harán falta para poder trabajar con las fórmulas.

```elm {l id="imports"}

import LogicUS.FOL.SintaxSemantics exposing (..)
import Set exposing (Set)
import Dict exposing (Dict)

ffolRead : String -> FormulaFOL
ffolRead s = ffolReadExtraction <| ffolReadFromString s
```

$\boxed{\textit{Apartado 1}}$ Formalicemos los enunciados:

- $\psi_1$: Toda estrella tiene algún cuadrado a la derecha.

```elm {l m=("$$ " ++ (ffolToMathString psi1) ++ " $$") context="1"  follows="imports"}
psi1 : FormulaFOL
psi1 = ffolRead "!A[x](E(x) -> !E[y](C(y) & D(y,x)))"
```

- $\psi_2$: Si a y b son dos posiciones de la cinta (no necesariamente distintas) entonces hay algún elemento que está a la izquierda de ambos y que no es un triángulo. 

```elm {l m=("$$ " ++ (ffolToMathString psi2) ++ " $$") context="1"}
psi2 : FormulaFOL
psi2 = ffolRead "!E[x](¬T(x) & I(x,*a) & I(x,*b))"
```

- $\psi_3$: En la cinta aparece la secuencia 	■ ★ ▲


```elm {l m=("$$ " ++ (ffolToMathString psi3) ++ " $$") context="1"}
psi3 : FormulaFOL
psi3 = ffolRead "!E[x] !E[y] !E[z] (C(x) & E(y) & T(z) & I(x, y) & I(y, z) & ¬!E[t] ( I(t,y) & D(t, x) | I(t,z) & D(t,y) ) )"
```

$\boxed{\textit{Apartado 2}}$ Comenzamos definiendo la fórmula:

```elm {l m=("$$ " ++ (ffolToMathString f) ++ " $$") context="2" follows="imports"}
f : FormulaFOL
f = ffolRead "!A[x] !E[y] (I(x,y) -> D(z,x) | !E[z] (D(z,y) & I(x, y))) "
```

La fórmula está bien formada ya que no existe ninguna variable cuantifcada por más de un cuantificador.

  ```elm {l r=(fIsWFF) context="2"}
  fIsWFF : Bool
  fIsWFF = ffolIsWellFormed f
  ```

<br> Estudiemos las variables libres y ligadas de la fórmula. Recordemos que una variable es libre si existe una instancia libre de esa fórmula. De forma que:

- Son variables libres:

  ```elm {l m=("$ " ++ (varsToMathString fvars) ++ " $") context="2"}
  fvars : List Variable
  fvars = ffolFreeVars f
  ```

- Son variables ligadas:

  ```elm {l m=("$ " ++ (varsToMathString lvars) ++ " $") context="2"}
  lvars : List Variable
  lvars = ffolLinkedVars f
  ```

El árbol de formación correspondería a:

```elm {l r=(fFTree) context="2"}
fFTree : String
fFTree = formTreeToDOT <| ffolFormTree f
```

<br> Representándolo:

<center>

```dot
digraph G {
rankdir=TB
graph []
node [shape=plaintext, color=black]
edge [dir=none, color=blue]

0 -> 1
1 -> 2
2 -> 3
2 -> 4
4 -> 5
4 -> 6
6 -> 7
7 -> 8
7 -> 9

0 [label="∀x∃y(I(x,y)→(D(z,x)∨∃z(D(z,y)∧I(x,y))))"]
1 [label="∃y(I(x,y)→(D(z,x)∨∃z(D(z,y)∧I(x,y))))"]
2 [label="(I(x,y)→(D(z,x)∨∃z(D(z,y)∧I(x,y))))"]
3 [label="I(x,y)"]
4 [label="(D(z,x)∨∃z(D(z,y)∧I(x,y)))"]
5 [label="D(z,x)"]
6 [label="∃z(D(z,y)∧I(x,y))"]
7 [label="(D(z,y)∧I(x,y))"]
8 [label="D(z,y)"]
9 [label="I(x,y)"]
}
```

</center>

Renombremos las variables:

```elm {l m=("$$" ++ (ffolToMathString f2) ++ "$$") context="2"}
f2 : FormulaFOL
f2 = renameVars <| f
```

<br> La fórmula no es cerrada

```elm {l r=(f2IsClosed) context="2"}
f2IsClosed : Bool
f2IsClosed = ffolIsClosed f2
```

<br> De forma que la clausura universal de la fórmula correspondería a:


```elm {l m=("$$ " ++ (ffolToMathString f3) ++ " $$") context="2"}
f3 : FormulaFOL
f3 =  ffolUniversalClausure f2
```

$\boxed{\textit{Apartado 3}}$. Vamos a definir las $L$-Estructuras asociadas a las cintas. 

$$ \begin{array}{c c c c c c} \hline M1 \hspace{0.5cm} & \Box & \Box &  \star  & \triangle & \Box \\ \hline & a & & & b & \end{array}$$

$$ \begin{array}{c c c c c c c} \hline M2 \hspace{0.5cm} & \triangle & \star &  \triangle  & \Box & \triangle & \triangle \\ \hline & & & & b & a &  \end{array}$$

$$ \begin{array}{c c c c c} \hline M3 \hspace{0.5cm} & \Box & \Box &  \Box  &  \Box \\ \hline & & \begin{matrix} a \\ b \end{matrix} \end{array}$$


Recuérdese que una $L$-estructura constaba de:

 - Un Universo $U$. En este caso el universo corresponderá a las posiciones. 

 - Una interpretación de las constantes a y b. En este caso las posiciones asociadas a los indicadores.

 - Una interpretación para los símbolos de función del lenguaje. No hay símbolos de función en $L$.

 - Una interpretación para cada predicado. Tomaremos el significado natural dado en el enunciado.

 De forma que en LogicUS debemos definir: 

 - El universo como un conjunto de elementos de la clase comparable (en nuestro caso enteros).

    ```elm {l context="3" follows="imports"}
    u1 : Set Int
    u1 = Set.fromList <| [1,2,3,4,5]

    u2 : Set Int
    u2 = Set.fromList <| [1,2,3,4,5,6]

    u3 : Set Int
    u3 = Set.fromList <| [1,2,3,4]
    ```

 - La interpretación de las constantes como un diccionario en el que a cada símbolo de constante le asignemos un elemento del dominio. 

    ```elm {l context="3"}
    ic1 : Dict Ident Int
    ic1 = Dict.fromList <| [(("a", []), 1), (("b", []), 4)]

    ic2 : Dict Ident Int
    ic2 = Dict.fromList <| [(("a", []), 5), (("b", []), 4)]

    ic3 : Dict Ident Int
    ic3 = Dict.fromList <| [(("a", []), 2), (("b", []), 2)]
    ```

- La interpretación de los predicados como un diccionario el que a cada símbolo de predicado se le asigna un natural correspondiente a su aridad y un conjunto de listas de elementos del universo, que son los que verifican el predicado

    ```elm {l context="3"}
    ip1 : Dict (String, List Int) (Int, Set (List Int))
    ip1 = 
      Dict.fromList <| 
        [ (("I", []), (2, Set.fromList [ [1,2], [1,3], [1,4], [1,5], [2,3], [2,4], [2,5], [3,4], [3,5], [4,5] ]))

          , (("D", []),(2, Set.fromList [[5,4], [5,3], [5,2], [5,1], [4,3], [4,2], [4,1], [3,2], [3,1], [2,1]]))

          , (("C", []), (1, Set.fromList [ [1], [2], [5] ] ) )

          , (("E", []), (1, Set.fromList [ [3] ] ))

          , (("T", []), (1, Set.fromList [ [4] ] ))      
        ]

    ip2 : Dict (String, List Int) (Int, Set (List Int))
    ip2 = 
      Dict.fromList <| 
        [ (("I", []), (2, Set.fromList <| List.concat <| List.map (\x -> List.map ( \ y -> [x, y]) (List.range (x+1) 6)) [1,2,3,4,5]))

          , (("D", []), (2, Set.fromList <| List.concat <| List.map (\x -> List.map ( \ y -> [x, y]) (List.range 1 (x-1))) [6,5,4,3,2]))

          , (("C", []), (1, Set.fromList [ [4] ] ) )

          , (("E", []), (1, Set.fromList [ [2] ] ))

          , (("T", []), (1, Set.fromList [ [1], [3], [5], [6] ] ))      
        ]

    ip3 : Dict (String, List Int) (Int, Set (List Int))
    ip3 = 
      Dict.fromList <| 
        [ (("I", []), (2, Set.fromList <| List.concat <| List.map (\x -> List.map ( \ y -> [x, y]) (List.range (x+1) 4)) [1,2,3]))

          , (("D", []), (2, Set.fromList <| List.concat <| List.map (\x -> List.map ( \ y -> [x, y]) (List.range 1 (x-1))) [4,3,2]))

          , (("C", []), (1, Set.fromList [ [1],[2],[3],[4] ] ) )

          , (("E", []), (1, Set.empty ))

          , (("T", []), (1, Set.empty ))      
        ]
    ```

De forma que ahora las $L$-estructuras serían:

```elm {l context="3"}
ls1 : L_Structure Int
ls1 = (u1, {const=ic1, func=Dict.empty, pred=ip1})

ls2 : L_Structure Int
ls2 = (u2, {const=ic2, func=Dict.empty, pred=ip2})


ls3 : L_Structure Int
ls3 = (u3, {const=ic3, func=Dict.empty, pred=ip3})
```

Ahora, para cada una de las fórmulas siguientes debemos decidir si alguna de las $L$-estructuras definidas la satisfacen.

  - $\varphi_1 \equiv C(a)  \vee (\neg E(b) \wedge (T(b) \rightarrow \exists x C(x)))$

    ```elm {l m=("$$ " ++ (ffolToMathString f1) ++ " $$") context="3"}
    f1 : FormulaFOL
    f1 = ffolRead "C(*a)|(¬E(*b) & (T(*b) -> !E[x] (C(x))))"
    ```
    La evaluamos respecto a cada una de las interpretaciones. 
    
    - Respecto a $M_1$

        ```elm {l r context="3"}
        evf1ls1 : Maybe Bool
        evf1ls1 = ffolValuation f1 ls1
        ```

    - Respecto a $M_2$

        ```elm {l r context="3"}
        evf1ls2 : Maybe Bool
        evf1ls2 = ffolValuation f1 ls2
        ```

    - Respecto a $M_3$

        ```elm {l r context="3"}
        evf1ls3 : Maybe Bool
        evf1ls3 = ffolValuation f1 ls3
        ```

<br>


  - $\varphi_2 \equiv \forall x \left( C(x) \rightarrow \exists y (T(y) \wedge I(y,x)) \right)$

    ```elm {l m=("$$ " ++ (ffolToMathString f2) ++ " $$") context="3"}
    f2 : FormulaFOL
    f2 = ffolRead "!A[x](C(x) -> !E[y](T(y) & I(y,x)))"
    ```
    La evaluamos respecto a cada una de las interpretaciones. 
    
    - Respecto a $M_1$

        ```elm {l r context="3"}
        evf2ls1 : Maybe Bool
        evf2ls1 = ffolValuation f2 ls1
        ```

    - Respecto a $M_2$

        ```elm {l r context="3"}
        evf2ls2 : Maybe Bool
        evf2ls2 = ffolValuation f2 ls2
        ```

    - Respecto a $M_3$

        ```elm {l r context="3"}
        evf2ls3 : Maybe Bool
        evf2ls3 = ffolValuation f2 ls3
        ```





