---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../../src
---

# Félix y el TX-150

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

Félix tiene en su garaje un taller que utiliza para la reparación de máquinas. Un buen amigo suyo va a hacerle una visita y obtiene la siguiente información:

- Todos los ordenadores son máquinas.

- El TX-150 es un ordenador

- Para cualquier máquina, Félix sabe sólo cómo estropearla o sólo cómo arreglarla.

- Toda máquina puede ser arreglada por alguien (incluso por otras máquinas), pero una máquina no puede arreglarse por sí misma.

- Las cosas sólo desesperan a quienes no son capaces de arreglarlas.

- Félix está deseperado con el TX-150.

Entonces, ¿puede estropear Félix el TX-150?

## Solución

Formalicemos el problema en el ámbito de la Lógica de Primer Orden. Para ello consideramos:

- Los símbolos de predicado: $O(x)$: "_$x$ es un ordenador_", $M(x)$: "_$x$ es una máquina_", $A(x,y)$: "_$x$ puede arreglar $y$_", $E(x,y)$: "_$x$ puede estropear $y$_" y $D(x, y)$: "_$x$ desespera a $y$_"

- Y los símbolos de constante $a$ y $b$ para representar al TX-150 y a Félix, respectivamente.

Antes de nada, vamos a importar el módulo de Sintaxis y Semántica en Primer Orden y algunos otros paquetes que nos harán falta para poder trabajar con las fórmulas.

Entonces las fórmulas asociadas a cada uno de los asertos corresponden a:

```elm {l id="imports"}

import LogicUS.FOL.SintaxSemantics exposing (..)
import LogicUS.FOL.Clauses exposing (..)
import LogicUS.FOL.Resolution exposing (..)
import LogicUS.FOL.NormalForms exposing (..)
import Set exposing (Set)
import Dict exposing (Dict)

ffolRead : String -> FormulaFOL
ffolRead s = ffolReadExtraction <| ffolReadFromString s
```

```elm {l m=("$ " ++ (sfolToMathString2 [psi1, psi2, psi3, psi4, psi5, psi6]) ++ " $") context="1"  follows="imports"}
psi1 : FormulaFOL
psi1 = ffolRead "!A[x](O(x) -> M(x))"

psi2 : FormulaFOL
psi2 = ffolRead "O(*a)"

psi3 : FormulaFOL
psi3 = ffolRead "!A[x](M(x) -> (A(*b, x) <-> ¬E(*b,x)))"

psi4 : FormulaFOL
psi4 = ffolRead "!A[x](M(x) -> (!E[y] A(y, x) & ¬A(x,x)))"

psi5 : FormulaFOL
psi5 = ffolRead "!A[x]!A[y](D(x,y) <-> ¬A(y,x))"

psi6 : FormulaFOL
psi6 = ffolRead "D(*a,*b)"
```

Ahora veámos si de la información anterior podemos deducir que Félix ha de poder estropear el TX-150. Esto es si $\Psi \models E(b,a)$.

```elm {l m=("$ " ++ (ffolToMathString psi7) ++ " $"), context="1"}
psi7 : FormulaFOL
psi7 = ffolRead "E(*b, *a)"
```

Para poder utilizar el método de resolución no restringida, primero hemos de pasar las fórmulas a forma Clausal. De forma que el conjunto de cláusulas asociadas a $\Psi \cup \{\neg E(*b, *a)\}$ corresponde a

 <!-- r=("$ " ++ (csfolToString cs) ++ " $") -->

```elm {l  context="1"}
cs : ClauseFOLSet
cs = sfolToClauses [psi1, psi2, psi3, psi4, psi5, psi6, Neg psi7]
```

$ \{ \{M(x),\neg O(x)\}, \, \{O(a)\}, \, \{\neg A(b,x),\neg E(b,x),\neg M(x)\}, \, \{A(b,x),E(b,x),\neg M(x)\}, \\ \{A(ş₁(x),x),\neg M(x)\}, \, \{\neg A(x,x),\neg M(x)\}, \, \{\neg A(y,x),\neg D(x,y)\}, \, \{A(y,x),D(x,y)\}, \\ \{D(a,b)\}, \, \{\neg E(b,a)\}\} $

Una vez obtenida el conjunto de cláusulas podemos aplicar la Resolución no restringida en Primer Orden como sigue. Comprobamos que es insatisfactible:

```elm {l r=(Tuple.first res) context="1"}
res : ( Bool, ResolutionTableau )
res = csfolSCFResolution cs
```

Mostrando el proceso haciendo uso de la orden `(resolutionTableauToDOT << Tuple.second) res` y eliminando las cláusulas irrelevantes:

^^^elm{r=((resolutionTableauToDOT << Tuple.second) res) context="1"}^^^^

```dot
digraph G {
 rankdir=TB
 graph []
 node [shape=box, color=white, fontcolor=black]
 edge [dir=none, color=blue, fontcolor=blue]

 1 -> 5
 2 -> 12
 3 -> 27
 4 -> 5 [label=<{x/a}>]
 5 -> 26
 10 -> 12 [xlabel=<{x/a, y/b}>]
 12 -> 25
 24 -> 25 [xlabel=<{x/a} >]
 25 -> 26
 26 -> 27

 1 [label=<{O(a)}>]
 2 [label=<{D(a,b)}>]
 3 [label=<{¬E(b,a)}>]
 4 [label=<{M(x),¬O(x)}>]
 5 [label=<{M(a)}>]
 10 [label=<{¬A(y,x),¬D(x,y)}>]
 12 [label=<{¬A(b,a)}>]
 24 [label=<{A(b,x),E(b,x),¬M(x)}>]
 25 [label=<{E(b,a),¬M(a)}>]
 26 [label=<{E(b,a)}>]
 27 [label=<□>]

 {rank=same; 1;2;3;4;10;24}
}
```
