---
elm:
  dependencies:
    avh4/elm-fifo: latest
    elm/browser: latest
    elm/core: latest
    elm/html: latest
    elm/json: latest
    elm/parser: latest
    elm/regex: latest
    elm/time: latest
    elm/url: latest
    elm/virtual-dom: latest
    elm-community/graph: latest
    elm-community/intdict: latest
    elm-community/list-extra: latest
    elm-community/maybe-extra: latest
    elm-community/string-extra: latest

  source-directories:
    - ../src
---

# Problema de las 8 Reinas

> Fuente: Ejercios de Lógica Computacional
> (https://www.cs.us.es/~fsancho/?p=logica-informatica-2020-21)

El Problema de las N reinas es un problema clásico dentro de los Problemas de Satisfacción de Restricciones. El problema fue originalmente propuesto en 1848 por el ajedrecista Max Bezzel para el tablero de ajedrez y considerando 8 Reinas. Durante años, muchos matemáticos, incluyendo a Gauss y Cantor, han trabajado en él y lo han generalizado a N-reinas. El problema se enuncia como:

Hallar, si existe, una distribución de $N$ reinas en un tablero de ajedrez $N \times N$ de manera no haya dos reinas que se ataquen entre sí.

En este ejercicio trabajaremos el problema original considerando un tablero de ajedrez clásico $8 \times 8$ y 8 reinas a colocar en el mismo. Se pide formalizar el problema y resolviéndolo haciendo uso de fórmulas proposicionales (parametrizadas).


## Solución

Cualquiera que conozca el ajedrez sabe que dos reinas se atacan entre sí si ocupan la misma fila, la misma columna o la misma diagonal. Por tanto para que el problema tenga solución se debe cumplir que para cada pareja de reinas se tiene que ambas ocupan filas, columnas y diagonales (principal y secundaria) distintas. Formalicémos el problema haciendo uso de fórmulas parametrizadas y resolvámoslo haciendo uso de LogicUS.

Bien, consideremos el tablero

<center>
<div style="width:408px; height:321px">
<table width="408px" height="321px"; border="0" cellspacing="2" cellpadding="2" bgcolor="#000000">
<tr align="center">
<td><font color="#ffffff">p<sub>81</sub></font></td>
<td bgcolor="#ffffff">p<sub>82</sub></td>
<td><font color="#ffffff">p<sub>83</sub></font></td>
<td bgcolor="#ffffff">p<sub>84</sub></td>
<td><font color="#ffffff">p<sub>85</sub></font></td>
<td bgcolor="#ffffff">p<sub>86</sub></td>
<td><font color="#ffffff">p<sub>87</sub></font></td>
<td bgcolor="#ffffff">p<sub>88</sub></td>
</tr>
<tr align="center">
<td bgcolor="#ffffff">p<sub>71</sub></td>
<td><font color="#ffffff">p<sub>72</sub></font></td>
<td bgcolor="#ffffff">p<sub>73</sub></td>
<td><font color="#ffffff">p<sub>74</sub></font></td>
<td bgcolor="#ffffff">p<sub>75</sub></td>
<td><font color="#ffffff">p<sub>76</sub></font></td>
<td bgcolor="#ffffff">p<sub>77</sub></td>
<td><font color="#ffffff">p<sub>78</sub></font></td>
<tr align="center">
<td><font color="#ffffff">p<sub>61</sub></font></td>
<td bgcolor="#ffffff">p<sub>62</sub></td>
<td><font color="#ffffff">p<sub>63</sub></font></td>
<td bgcolor="#ffffff">p<sub>64</sub></td>
<td><font color="#ffffff">p<sub>65</sub></font></td>
<td bgcolor="#ffffff">p<sub>66</sub></td>
<td><font color="#ffffff">p<sub>67</sub></font></td>
<td bgcolor="#ffffff">p<sub>68</sub></td>
</tr>
<tr align="center">
<td bgcolor="#ffffff">p<sub>51</sub></td>
<td><font color="#ffffff">p<sub>52</sub></font></td>
<td bgcolor="#ffffff">p<sub>53</sub></td>
<td><font color="#ffffff">p<sub>54</sub></font></td>
<td bgcolor="#ffffff">p<sub>55</sub></td>
<td><font color="#ffffff">p<sub>56</sub></font></td>
<td bgcolor="#ffffff">p<sub>57</sub></td>
<td><font color="#ffffff">p<sub>58</sub></font></td>
</tr>  <tr align="center">
<td><font color="#ffffff">p<sub>41</sub></font></td>
<td bgcolor="#ffffff">p<sub>42</sub></td>
<td><font color="#ffffff">p<sub>43</sub></font></td>
<td bgcolor="#ffffff">p<sub>44</sub></td>
<td><font color="#ffffff">p<sub>45</sub></font></td>
<td bgcolor="#ffffff">p<sub>46</sub></td>
<td><font color="#ffffff">p<sub>47</sub></font></td>
<td bgcolor="#ffffff">p<sub>48</sub></td>
</tr>
<tr align="center">
<td bgcolor="#ffffff">p<sub>31</sub></td>
<td><font color="#ffffff">p<sub>32</sub></font></td>
<td bgcolor="#ffffff">p<sub>33</sub></td>
<td><font color="#ffffff">p<sub>34</sub></font></td>
<td bgcolor="#ffffff">p<sub>35</sub></td>
<td><font color="#ffffff">p<sub>36</sub></font></td>
<td bgcolor="#ffffff">p<sub>37</sub></td>
<td><font color="#ffffff">p<sub>38</sub></font></td>
</tr>  <tr align="center">
<td><font color="#ffffff">p<sub>21</sub></font></td>
<td bgcolor="#ffffff">p<sub>22</sub></td>
<td><font color="#ffffff">p<sub>23</sub></font></td>
<td bgcolor="#ffffff">p<sub>24</sub></td>
<td><font color="#ffffff">p<sub>25</sub></font></td>
<td bgcolor="#ffffff">p<sub>26</sub></td>
<td><font color="#ffffff">p<sub>27</sub></font></td>
<td bgcolor="#ffffff">p<sub>28</sub></td>
</tr>  <tr align="center">
<td bgcolor="#ffffff">p<sub>11</sub></td>
<td><font color="#ffffff">p<sub>12</sub></font></td>
<td bgcolor="#ffffff">p<sub>13</sub></td>
<td><font color="#ffffff">p<sub>14</sub></font></td>
<td bgcolor="#ffffff">p<sub>15</sub></td>
<td><font color="#ffffff">p<sub>16</sub></font></td>
<td bgcolor="#ffffff">p<sub>17</sub></td>
<td><font color="#ffffff">p<sub>18</sub></font></td>
</tr>
</table>
</div>
</center>

<br>

```elm {l id="imports"}
import LogicUS.PL.CSP exposing (..)
import LogicUS.PL.SintaxSemantics exposing (..)

bfplRead : String -> BigFPL
bfplRead = bfplReadExtraction << bfplReadFromString
```

<br> Bien, ténganse las variables proposicionales $P_{i,j}$ con $i,j \in \{1,\dots,8\}$ tal que $P_{i,j}$ vale 1 si una reina está colocada en la posición $p_{i,j}$.


Entonces las restricciones pueden modelarse como:

- Dos reinas no ocupan una misma fila. Esto es en toda fila se cumple que si una casilla de la fila tiene una reina entonces las demás no pueden tener reina:

```elm {l m=("$$" ++ (bfplToMathString f1) ++ "$$") context="1"  follows="imports"}
f1 : BigFPL
f1 = bfplRead "!&[i(1:8)]{T} (!&[j(1:8)]{T} (P_{i,j} -> !&[k(1:8)]{[k!=j]}(¬P_{i,k})))"
```

- Dos reinas no ocupan una misma columna. Esto es en toda columna se cumple que si una casilla de la columna tiene una reina entonces las demás no pueden tener reina:

```elm {l m=("$$" ++ (bfplToMathString f2) ++ "$$") context="1"}
f2 : BigFPL
f2 = bfplRead "!&[j(1:8)]{T} (!&[i(1:8)]{T} (P_{i,j} -> !&[k(1:8)]{[k!=i]}(¬P_{k,j})))"
```

- Dos reinas no ocupan una misma diagonal principal. Esto es en toda casilla se cumple que si esa casilla contiene una reina entonces las demás casillas de esa diagonal principal no pueden tener una reina:

```elm {l context="1"}
f3 : BigFPL
f3 = bfplRead "!&[i(1:8)]{T} (!&[j(1:8)]{T} (P_{i,j} -> !&[k(-7:7)]{[k!=0]AND[i+k>0]AND[i+k<=8]AND[j+k>0]AND[j+k<=8]}(¬P_{i+k,j+k})))"
```

^^^elm{m=("$$" ++ (Tuple.first <| bfplToMathString2 f3) ++ "$$") context="1"}^^^

^^^elm{m=("$$" ++ (Tuple.second <| bfplToMathString2 f3) ++ "$$") context="1"}^^^

- Dos reinas no ocupan una misma diagonal principal. Esto es en toda casilla se cumple que si esa casilla contiene una reina entonces las demás casillas de esa diagonal principal no pueden tener una reina:

```elm {l context="1"}
f4 : BigFPL
f4 = bfplRead "!&[i(1:8)]{T} (!&[j(1:8)]{T} (P_{i,j} -> !&[k(-7:7)]{[k!=0]AND[i+k>0]AND[i+k<=8]AND[j-k>0]AND[j-k<=8]}(¬P_{i+k,j-k})))"
```

^^^elm{m=("$$" ++ (Tuple.first <| bfplToMathString2 f4) ++ "$$") context="1"}^^^

^^^elm{m=("$$" ++ (Tuple.second <| bfplToMathString2 f4) ++ "$$") context="1"}^^^


- Han de colocarse 8 reinas. Esto es equivalente a decir que en cada fila ha de haber una reina. 

```elm {l m=("$$" ++ (bfplToMathString f5) ++ "$$") context="1"}
f5 : BigFPL
f5 = bfplRead "!&[i(1:8)]{T} (!|[j(1:8)]{T} (P_{i,j}))"
```

Resolvamos el problema. Esto es comprobemos si el conjunto es satisfactible y si lo es entonces encontremos un modelo. Esto es precisamente lo que posibilita la función `sbfplsolver`.

```elm {l context="1"}
sol : (Bool, Interpretation)
sol = sbfplsolver [f1,f2,f3,f4,f5]
```

^^^elm{m=("¿Es satisfactible?  " ++  (Debug.toString <|Tuple.first sol)) context="1"}^^^

^^^elm{m=("Un modelo: $" ++  (solutionModelToMathString <| Tuple.second sol) ++ "$" ) context="1"}^^^


Que correspondería en el tablero a:

<center>

<div style="width:408px; height:321px">
<table width="408px" height="321px"; border="0" cellspacing="2" cellpadding="2" bgcolor="#000000">
<tr align="center">
<td><font color="#ffffff">p<sub>81</sub></font></td>
<td bgcolor="#ffffff">p<sub>82</sub></td>
<td><font color="#ffffff">p<sub>83</sub></font></td>
<td bgcolor="#ffffff">p<sub>84</sub></td>
<td><font color="#ffffff">p<sub>85</sub></font></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>86</sub></td>
<td><font color="#ffffff">p<sub>87</sub></font></td>
<td bgcolor="#ffffff">p<sub>88</sub></td>
</tr>
<tr align="center">
<td bgcolor="#ffffff">p<sub>71</sub></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>72</sub></font></td>
<td bgcolor="#ffffff">p<sub>73</sub></td>
<td><font color="#ffffff">p<sub>74</sub></font></td>
<td bgcolor="#ffffff">p<sub>75</sub></td>
<td><font color="#ffffff">p<sub>76</sub></font></td>
<td bgcolor="#ffffff">p<sub>77</sub></td>
<td><font color="#ffffff">p<sub>78</sub></font></td>
<tr align="center">
<td><font color="#ffffff">p<sub>61</sub></font></td>
<td bgcolor="#ffffff">p<sub>62</sub></td>
<td><font color="#ffffff">p<sub>63</sub></font></td>
<td bgcolor="#ffffff">p<sub>64</sub></td>
<td><font color="#ffffff">p<sub>65</sub></font></td>
<td bgcolor="#ffffff">p<sub>66</sub></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>67</sub></font></td>
<td bgcolor="#ffffff">p<sub>68</sub></td>
</tr>
<tr align="center">
<td bgcolor="#00ff00"><font color="#008f39">p<sub>51</sub></td>
<td><font color="#ffffff">p<sub>52</sub></font></td>
<td bgcolor="#ffffff">p<sub>53</sub></td>
<td><font color="#ffffff">p<sub>54</sub></font></td>
<td bgcolor="#ffffff">p<sub>55</sub></td>
<td><font color="#ffffff">p<sub>56</sub></font></td>
<td bgcolor="#ffffff">p<sub>57</sub></td>
<td><font color="#ffffff">p<sub>58</sub></font></td>
</tr>  <tr align="center">
<td><font color="#ffffff">p<sub>41</sub></font></td>
<td bgcolor="#ffffff">p<sub>42</sub></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>43</sub></font></td>
<td bgcolor="#ffffff">p<sub>44</sub></td>
<td><font color="#ffffff">p<sub>45</sub></font></td>
<td bgcolor="#ffffff">p<sub>46</sub></td>
<td><font color="#ffffff">p<sub>47</sub></font></td>
<td bgcolor="#ffffff">p<sub>48</sub></td>
</tr>
<tr align="center">
<td bgcolor="#ffffff">p<sub>31</sub></td>
<td><font color="#ffffff">p<sub>32</sub></font></td>
<td bgcolor="#ffffff">p<sub>33</sub></td>
<td><font color="#ffffff">p<sub>34</sub></font></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>35</sub></td>
<td><font color="#ffffff">p<sub>36</sub></font></td>
<td bgcolor="#ffffff">p<sub>37</sub></td>
<td><font color="#ffffff">p<sub>38</sub></font></td>
</tr>  <tr align="center">
<td><font color="#ffffff">p<sub>21</sub></font></td>
<td bgcolor="#ffffff">p<sub>22</sub></td>
<td><font color="#ffffff">p<sub>23</sub></font></td>
<td bgcolor="#ffffff">p<sub>24</sub></td>
<td><font color="#ffffff">p<sub>25</sub></font></td>
<td bgcolor="#ffffff">p<sub>26</sub></td>
<td><font color="#ffffff">p<sub>27</sub></font></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>28</sub></td>
</tr>  <tr align="center">
<td bgcolor="#ffffff">p<sub>11</sub></td>
<td><font color="#ffffff">p<sub>12</sub></font></td>
<td bgcolor="#ffffff">p<sub>13</sub></td>
<td bgcolor="#00ff00"><font color="#008f39">p<sub>14</sub></font></td>
<td bgcolor="#ffffff">p<sub>15</sub></td>
<td><font color="#ffffff">p<sub>16</sub></font></td>
<td bgcolor="#ffffff">p<sub>17</sub></td>
<td><font color="#ffffff">p<sub>18</sub></font></td>
</tr>
</table>
</div>

</center>


